package service;

import java.util.Date;
import net.jini.core.event.*;
import java.util.Vector;
import java.rmi.RemoteException;

public class TickerTimer implements Timer {
    private Date time;
    private boolean isValid;
    private Ticker ticker;
    private Vector listeners = new Vector();
    private int seqNum = 0;  // event sequence counter

    /**
     * Constructor with no starting time has
     * invalid timer and any time
     */
    public TickerTimer() {
	time = new Date(0);
	isValid = false;
	ticker = new Ticker(time);
	ticker.start();
	fireNotify(INVALID, seqNum++);  // valid has changed state
    }

    public TickerTimer(Date t) {
	time = t;
	isValid = true;
	ticker = new Ticker(time);
	ticker.start();
	fireNotify(VALID, seqNum++);	// valid has changed state
    }
 
    public void setTime(Date t) {
	System.out.println("Setting time to " + t);
	time = t;
	isValid = true;
	if (ticker != null) {
	    ticker.stopRunning();
	}
	ticker = new Ticker(time);
	ticker.start();
	fireNotify(VALID, seqNum++);	// valid has changed state
    }

    public Date getTime() {
	return ticker.getTime();
    }

    public boolean isValidTime() {
	if (isValid) {
	    return true;
	} else {
	    return false;
	}
    }

    public EventRegistration addEventListener(RemoteEventListener listener) {
	listeners.add(listener);
	return new EventRegistration(0L, this, null, 0L);
    }

    protected void fireNotify(long eventID,
			      long seqNum) {
	RemoteEvent remoteEvent = new RemoteEvent(this, eventID, 
						  seqNum, null);
	for (int n = 0; n < listeners.size(); n++) {
	    RemoteEventListener listener = 
		(RemoteEventListener) listeners.elementAt(n);
	    try {
		listener.notify(remoteEvent);
	    } catch(UnknownEventException e) {
		// ignore
	    } catch(RemoteException e) {
		// ignore
	    }
	}
    }
}

class Ticker extends Thread {
    private Date time;
    private boolean keepRunning = true;

    public Ticker(Date t) {
	time = t;
    }

    public Date getTime() {
	return time;
    }

    public void run() {
	while (keepRunning) {
	    try {
		sleep(1000);
	    } catch(InterruptedException e) {
	    }
	    time = new Date(time.getTime() + 1000);
	}
    }

    public void stopRunning() {
	keepRunning = false;
    }
}
